@@ -198,7 +198,7 @@ public class FragmentHistory extends Fragment implements IAdapterRefreshable {
         Log.d(TAG,"fin save");
 
         if (to_remove.size() > 0) {
-            Toast toast = Toast.makeText(getContext(), getResources().getString(R.string.notify_sync_list_deleted_entries, to_remove.size(), historyManager.size()), Toast.LENGTH_SHORT);
+            Toast toast = Toast.makeText(getContext(), getResources().getString(R.string.notify_sync_list_deleted_entries, to_remove.size(), historyManager.size()), Toast.LENGTH_LONG);
             toast.show();
         }
     }
