@@ -124,7 +124,7 @@ public class FragmentStarred extends Fragment implements IAdapterRefreshable, Ob
             @Override
             public void onStationMoveFinished() {
                 // We don't want to update RecyclerView during its layout process
-                Objects.requireNonNull(getView()).post(() -> {
+                requireView().post(() -> {
                     favouriteManager.Save();
                     favouriteManager.notifyObservers();
                 });
@@ -215,7 +215,7 @@ public class FragmentStarred extends Fragment implements IAdapterRefreshable, Ob
         Log.d(TAG,"fin save");
 
         if (to_remove.size() > 0) {
-            Toast toast = Toast.makeText(getContext(), getResources().getString(R.string.notify_sync_list_deleted_entries, to_remove.size(), favouriteManager.size()), Toast.LENGTH_SHORT);
+            Toast toast = Toast.makeText(getContext(), getResources().getString(R.string.notify_sync_list_deleted_entries, to_remove.size(), favouriteManager.size()), Toast.LENGTH_LONG);
             toast.show();
         }
     }
