@@ -60,8 +60,7 @@ public class PlayStationTask extends AsyncTask<Void, Void, String> {
     public static PlayStationTask playExternal(DataRadioStation stationToPlay, Context ctx) {
         return new PlayStationTask(stationToPlay, ctx, url -> {
             Intent share = new Intent(Intent.ACTION_VIEW);
-            share.setType("audio/*");
-            share.setData(Uri.parse(url));
+            share.setDataAndType(Uri.parse(url), "audio/*");
             ctx.startActivity(share);
         }, null);
     }
