@@ -11,32 +11,26 @@ import android.net.Uri;
 import android.os.Handler;
 import android.os.Looper;
 import android.util.Log;
-import android.view.Surface;
 
 import androidx.annotation.NonNull;
 import androidx.annotation.Nullable;
 import androidx.preference.PreferenceManager;
 
 import com.google.android.exoplayer2.C;
-import com.google.android.exoplayer2.ExoPlaybackException;
-import com.google.android.exoplayer2.Format;
+import com.google.android.exoplayer2.ExoPlayer;
+import com.google.android.exoplayer2.MediaItem;
+import com.google.android.exoplayer2.PlaybackException;
 import com.google.android.exoplayer2.PlaybackParameters;
 import com.google.android.exoplayer2.Player;
-import com.google.android.exoplayer2.SimpleExoPlayer;
 import com.google.android.exoplayer2.analytics.AnalyticsListener;
 import com.google.android.exoplayer2.audio.AudioAttributes;
-import com.google.android.exoplayer2.decoder.DecoderCounters;
 import com.google.android.exoplayer2.metadata.Metadata;
-import com.google.android.exoplayer2.metadata.MetadataOutput;
 import com.google.android.exoplayer2.metadata.icy.IcyHeaders;
 import com.google.android.exoplayer2.metadata.icy.IcyInfo;
 import com.google.android.exoplayer2.metadata.id3.Id3Frame;
 import com.google.android.exoplayer2.source.MediaSource;
-import com.google.android.exoplayer2.source.MediaSourceEventListener;
 import com.google.android.exoplayer2.source.ProgressiveMediaSource;
-import com.google.android.exoplayer2.source.TrackGroupArray;
 import com.google.android.exoplayer2.source.hls.HlsMediaSource;
-import com.google.android.exoplayer2.trackselection.TrackSelectionArray;
 import com.google.android.exoplayer2.upstream.DataSource;
 import com.google.android.exoplayer2.upstream.DefaultBandwidthMeter;
 import com.google.android.exoplayer2.upstream.DefaultLoadErrorHandlingPolicy;
@@ -46,10 +40,10 @@ import net.programmierecke.radiodroid2.BuildConfig;
 import net.programmierecke.radiodroid2.R;
 import net.programmierecke.radiodroid2.Utils;
 import net.programmierecke.radiodroid2.players.PlayState;
+import net.programmierecke.radiodroid2.players.PlayerWrapper;
 import net.programmierecke.radiodroid2.recording.RecordableListener;
 import net.programmierecke.radiodroid2.station.live.ShoutcastInfo;
 import net.programmierecke.radiodroid2.station.live.StreamLiveInfo;
-import net.programmierecke.radiodroid2.players.PlayerWrapper;
 
 import java.io.IOException;
 import java.util.HashMap;
@@ -57,16 +51,16 @@ import java.util.Map;
 
 import okhttp3.OkHttpClient;
 
-public class ExoPlayerWrapper implements PlayerWrapper, IcyDataSource.IcyDataSourceListener, MetadataOutput {
+public class ExoPlayerWrapper implements PlayerWrapper, IcyDataSource.IcyDataSourceListener, Player.Listener {
 
     final private String TAG = "ExoPlayerWrapper";
 
-    private SimpleExoPlayer player;
+    private ExoPlayer player;
     private PlayListener stateListener;
 
     private String streamUrl;
 
-    private DefaultBandwidthMeter bandwidthMeter = new DefaultBandwidthMeter();
+    private final DefaultBandwidthMeter bandwidthMeter = new DefaultBandwidthMeter();
 
     private RecordableListener recordableListener;
 
@@ -91,68 +85,17 @@ public class ExoPlayerWrapper implements PlayerWrapper, IcyDataSource.IcyDataSou
 
                 cancelStopTask();
 
-                player.prepare(audioSource);
+                player.setMediaSource(audioSource);
+                player.prepare();
                 player.setPlayWhenReady(true);
             }
         }
     };
-    final class CustomLoadErrorHandlingPolicy extends DefaultLoadErrorHandlingPolicy {
-        final int MIN_RETRY_DELAY_MS = 10;
-        final SharedPreferences sharedPrefs = PreferenceManager.getDefaultSharedPreferences(context);
-
-        // We need to read the retry delay here on each error again because the user might change
-        // this value between retries and experiment with different vales to get the best result for
-        // the specific situation. We also need to make sure that a sensible minimum value is chosen.
-        int getSanitizedRetryDelaySettingsMs() {
-            return Math.max(sharedPrefs.getInt("settings_retry_delay", 100), MIN_RETRY_DELAY_MS);
-        }
-
-        @Override
-        public long getRetryDelayMsFor(
-                int dataType,
-                long loadDurationMs,
-                IOException exception,
-                int errorCount) {
-
-            int retryDelay = getSanitizedRetryDelaySettingsMs();
-
-            if (exception instanceof HttpDataSource.InvalidContentTypeException) {
-                stateListener.onPlayerError(R.string.error_play_stream);
-                return C.TIME_UNSET; // Immediately surface error if we cannot play content type
-            }
-
-            if (!Utils.hasAnyConnection(context)) {
-                int resumeWithinS = sharedPrefs.getInt("settings_resume_within", 60);
-                if (resumeWithinS > 0) {
-                    resumeWhenNetworkConnected();
-                    retryDelay = 1000 * resumeWithinS + retryDelay;
-                }
-            }
-
-            if (BuildConfig.DEBUG) {
-                Log.d(TAG, "Providing retry delay of " + retryDelay + "ms " +
-                        "for: data type " + dataType + ", " +
-                        "load duration: " + loadDurationMs + "ms, " +
-                        "error count: " + errorCount + ", " +
-                        "exception " + exception.getClass() + ", " +
-                        "message: " + exception.getMessage());
-            }
-            return retryDelay;
-        }
-
-        @Override
-        public int getMinimumLoadableRetryCount(int dataType) {
-            return sharedPrefs.getInt("settings_retry_timeout", 10) * 1000 / getSanitizedRetryDelaySettingsMs() + 1;
-        }
-    }
 
     @Override
     public void playRemote(@NonNull OkHttpClient httpClient, @NonNull String streamUrl, @NonNull Context context, boolean isAlarm) {
         // I don't know why, but it is still possible that streamUrl is null,
         // I still get exceptions from this from google
-        if (streamUrl == null) {
-            return;
-        }
         if (!streamUrl.equals(this.streamUrl)) {
             currentPlaybackTransferredBytes = 0;
         }
@@ -169,13 +112,12 @@ public class ExoPlayerWrapper implements PlayerWrapper, IcyDataSource.IcyDataSou
         }
 
         if (player == null) {
-            player = new SimpleExoPlayer.Builder(context).build();
-            player.setAudioAttributes(new AudioAttributes.Builder().setContentType(C.CONTENT_TYPE_MUSIC)
-                    .setUsage(isAlarm ? C.USAGE_ALARM : C.USAGE_MEDIA).build());
+            player = new ExoPlayer.Builder(context).build();
+            player.setAudioAttributes(new AudioAttributes.Builder().setContentType(C.AUDIO_CONTENT_TYPE_MUSIC)
+                    .setUsage(isAlarm ? C.USAGE_ALARM : C.USAGE_MEDIA).build(), false);
 
-            player.addListener(new ExoPlayerListener());
+            player.addListener(this);
             player.addAnalyticsListener(new AnalyticEventListener());
-            player.addMetadataOutput(this);
         }
 
         if (playerThreadHandler == null) {
@@ -193,13 +135,15 @@ public class ExoPlayerWrapper implements PlayerWrapper, IcyDataSource.IcyDataSou
         if (!isHls) {
             audioSource = new ProgressiveMediaSource.Factory(dataSourceFactory)
                     .setLoadErrorHandlingPolicy(new CustomLoadErrorHandlingPolicy())
-                    .createMediaSource(Uri.parse(streamUrl));
-            player.prepare(audioSource);
+                    .createMediaSource(MediaItem.fromUri(Uri.parse(streamUrl)));
+            player.setMediaSource(audioSource);
+            player.prepare();
         } else {
             audioSource = new HlsMediaSource.Factory(dataSourceFactory)
                     .setLoadErrorHandlingPolicy(new CustomLoadErrorHandlingPolicy())
-                    .createMediaSource(Uri.parse(streamUrl));
-            player.prepare(audioSource);
+                    .createMediaSource(MediaItem.fromUri(Uri.parse(streamUrl)));
+            player.setMediaSource(audioSource);
+            player.prepare();
         }
 
         player.setPlayWhenReady(true);
@@ -299,34 +243,32 @@ public class ExoPlayerWrapper implements PlayerWrapper, IcyDataSource.IcyDataSou
     }
 
     @Override
-    public void onMetadata(Metadata metadata) {
-        if (BuildConfig.DEBUG) Log.d(TAG, "META: " + metadata.toString());
-        if ((metadata != null)) {
-            final int length = metadata.length();
-            if (length > 0) {
-                for (int i = 0; i < length; i++) {
-                    final Metadata.Entry entry = metadata.get(i);
-                    if (entry == null) {
-                        continue;
-                    }
-                    if (entry instanceof IcyInfo) {
-                        final IcyInfo icyInfo = ((IcyInfo) entry);
-                        Log.d(TAG, "IcyInfo: " + icyInfo.toString());
-                        if (icyInfo.title != null) {
-                            Map<String, String> rawMetadata = new HashMap<String, String>() {{
-                                put("StreamTitle", icyInfo.title);
-                            }};
-                            StreamLiveInfo streamLiveInfo = new StreamLiveInfo(rawMetadata);
-                            onDataSourceStreamLiveInfo(streamLiveInfo);
-                        }
-                    } else if (entry instanceof IcyHeaders) {
-                        final IcyHeaders icyHeaders = ((IcyHeaders) entry);
-                        Log.d(TAG, "IcyHeaders: " + icyHeaders.toString());
-                        onDataSourceShoutcastInfo(new ShoutcastInfo(icyHeaders));
-                    } else if (entry instanceof Id3Frame) {
-                        final Id3Frame id3Frame = ((Id3Frame) entry);
-                        Log.d(TAG, "id3 metadata: " + id3Frame.toString());
+    public void onMetadata(@NonNull Metadata metadata) {
+        if (BuildConfig.DEBUG) Log.d(TAG, "META: " + metadata);
+        final int length = metadata.length();
+        if (length > 0) {
+            for (int i = 0; i < length; i++) {
+                final Metadata.Entry entry = metadata.get(i);
+                if (entry == null) {
+                    continue;
+                }
+                if (entry instanceof IcyInfo) {
+                    final IcyInfo icyInfo = ((IcyInfo) entry);
+                    Log.d(TAG, "IcyInfo: " + icyInfo);
+                    if (icyInfo.title != null) {
+                        Map<String, String> rawMetadata = new HashMap<>() {{
+                            put("StreamTitle", icyInfo.title);
+                        }};
+                        StreamLiveInfo streamLiveInfo = new StreamLiveInfo(rawMetadata);
+                        onDataSourceStreamLiveInfo(streamLiveInfo);
                     }
+                } else if (entry instanceof IcyHeaders) {
+                    final IcyHeaders icyHeaders = ((IcyHeaders) entry);
+                    Log.d(TAG, "IcyHeaders: " + icyHeaders);
+                    onDataSourceShoutcastInfo(new ShoutcastInfo(icyHeaders));
+                } else if (entry instanceof Id3Frame) {
+                    final Id3Frame id3Frame = ((Id3Frame) entry);
+                    Log.d(TAG, "id3 metadata: " + id3Frame);
                 }
             }
         }
@@ -357,7 +299,7 @@ public class ExoPlayerWrapper implements PlayerWrapper, IcyDataSource.IcyDataSou
 
                     ExoPlayerWrapper.this.fullStopTask = null;
                 };
-                playerThreadHandler.postDelayed(fullStopTask, resumeWithin * 1000);
+                playerThreadHandler.postDelayed(fullStopTask, resumeWithin * 1000L);
 
                 stateListener.onPlayerWarning(R.string.warning_no_network_trying_resume);
             } else {
@@ -428,46 +370,68 @@ public class ExoPlayerWrapper implements PlayerWrapper, IcyDataSource.IcyDataSou
         }
     }
 
-    private class ExoPlayerListener implements Player.EventListener {
+    @Override
+    public void onRepeatModeChanged(int repeatMode) {
+        // Do nothing
+    }
 
-        @Override
-        public void onTracksChanged(TrackGroupArray trackGroups, TrackSelectionArray trackSelections) {
-            // Do nothing
+    @Override
+    public void onPlayerErrorChanged(PlaybackException error) {
+        Log.d(TAG, "Player error: ", error);
+        // Stop playing since it is either irrecoverable error in the player or our data source failed to reconnect.
+        if (fullStopTask != null) {
+            stop();
+            stateListener.onPlayerError(R.string.error_play_stream);
+        }
+    }
+
+    @Override
+    public void onPlaybackParametersChanged(@NonNull PlaybackParameters playbackParameters) {
+        // Do nothing
+    }
+
+    final class CustomLoadErrorHandlingPolicy extends DefaultLoadErrorHandlingPolicy {
+        final int MIN_RETRY_DELAY_MS = 10;
+        final SharedPreferences sharedPrefs = PreferenceManager.getDefaultSharedPreferences(context);
+
+        // We need to read the retry delay here on each error again because the user might change
+        // this value between retries and experiment with different vales to get the best result for
+        // the specific situation. We also need to make sure that a sensible minimum value is chosen.
+        int getSanitizedRetryDelaySettingsMs() {
+            return Math.max(sharedPrefs.getInt("settings_retry_delay", 100), MIN_RETRY_DELAY_MS);
         }
 
         @Override
-        public void onLoadingChanged(boolean isLoading) {
-            // Do nothing
-        }
+        public long getRetryDelayMsFor(LoadErrorInfo loadErrorInfo) {
 
-        @Override
-        public void onPlayerStateChanged(boolean playWhenReady, int playbackState) {
-            // Do nothing
-        }
+            int retryDelay = getSanitizedRetryDelaySettingsMs();
+            IOException exception = loadErrorInfo.exception;
 
-        @Override
-        public void onRepeatModeChanged(int repeatMode) {
-            // Do nothing
-        }
-
-        @Override
-        public void onPlayerError(ExoPlaybackException error) {
-            Log.d(TAG, "Player error: ", error);
-            // Stop playing since it is either irrecoverable error in the player or our data source failed to reconnect.
-            if (fullStopTask != null || error.type != ExoPlaybackException.TYPE_SOURCE) {
-                stop();
+            if (exception instanceof HttpDataSource.InvalidContentTypeException) {
                 stateListener.onPlayerError(R.string.error_play_stream);
+                return C.TIME_UNSET; // Immediately surface error if we cannot play content type
             }
+
+            if (!Utils.hasAnyConnection(context)) {
+                int resumeWithinS = sharedPrefs.getInt("settings_resume_within", 60);
+                if (resumeWithinS > 0) {
+                    resumeWhenNetworkConnected();
+                    retryDelay = 1000 * resumeWithinS + retryDelay;
+                }
+            }
+
+            if (BuildConfig.DEBUG) {
+                Log.d(TAG, "Providing retry delay of " + retryDelay + "ms " +
+                        "error count: " + loadErrorInfo.errorCount + ", " +
+                        "exception " + exception.getClass() + ", " +
+                        "message: " + exception.getMessage());
+            }
+            return retryDelay;
         }
 
         @Override
-        public void onPositionDiscontinuity(int reason) {
-            // Do nothing
-        }
-
-        @Override
-        public void onPlaybackParametersChanged(PlaybackParameters playbackParameters) {
-            // Do nothing
+        public int getMinimumLoadableRetryCount(int dataType) {
+            return sharedPrefs.getInt("settings_retry_timeout", 10) * 1000 / getSanitizedRetryDelaySettingsMs() + 1;
         }
     }
 
@@ -489,192 +453,77 @@ public class ExoPlayerWrapper implements PlayerWrapper, IcyDataSource.IcyDataSou
         }
 
         @Override
-        public void onTimelineChanged(EventTime eventTime, int reason) {
+        public void onTimelineChanged(@NonNull EventTime eventTime, int reason) {
 
         }
 
         @Override
-        public void onPositionDiscontinuity(EventTime eventTime, int reason) {
+        public void onPlaybackParametersChanged(@NonNull EventTime eventTime, @NonNull PlaybackParameters playbackParameters) {
 
         }
 
         @Override
-        public void onSeekStarted(EventTime eventTime) {
+        public void onRepeatModeChanged(@NonNull EventTime eventTime, int repeatMode) {
 
         }
 
         @Override
-        public void onSeekProcessed(EventTime eventTime) {
+        public void onShuffleModeChanged(@NonNull EventTime eventTime, boolean shuffleModeEnabled) {
 
         }
 
         @Override
-        public void onPlaybackParametersChanged(EventTime eventTime, PlaybackParameters playbackParameters) {
+        public void onBandwidthEstimate(@NonNull EventTime eventTime, int totalLoadTimeMs, long totalBytesLoaded, long bitrateEstimate) {
 
         }
 
         @Override
-        public void onRepeatModeChanged(EventTime eventTime, int repeatMode) {
+        public void onSurfaceSizeChanged(@NonNull EventTime eventTime, int width, int height) {
 
         }
 
         @Override
-        public void onShuffleModeChanged(EventTime eventTime, boolean shuffleModeEnabled) {
+        public void onMetadata(@NonNull EventTime eventTime, @NonNull Metadata metadata) {
 
         }
 
         @Override
-        public void onLoadingChanged(EventTime eventTime, boolean isLoading) {
+        public void onAudioAttributesChanged(@NonNull EventTime eventTime, @NonNull AudioAttributes audioAttributes) {
 
         }
 
         @Override
-        public void onPlayerError(EventTime eventTime, ExoPlaybackException error) {
-            Log.d(TAG, "Player error at playback position " + eventTime.currentPlaybackPositionMs + "ms: ", error);
-        }
-
-        @Override
-        public void onTracksChanged(EventTime eventTime, TrackGroupArray trackGroups, TrackSelectionArray trackSelections) {
+        public void onVolumeChanged(@NonNull EventTime eventTime, float volume) {
 
         }
 
         @Override
-        public void onLoadStarted(EventTime eventTime, MediaSourceEventListener.LoadEventInfo loadEventInfo, MediaSourceEventListener.MediaLoadData mediaLoadData) {
+        public void onDroppedVideoFrames(@NonNull EventTime eventTime, int droppedFrames, long elapsedMs) {
 
         }
 
         @Override
-        public void onLoadCompleted(EventTime eventTime, MediaSourceEventListener.LoadEventInfo loadEventInfo, MediaSourceEventListener.MediaLoadData mediaLoadData) {
+        public void onDrmKeysLoaded(@NonNull EventTime eventTime) {
 
         }
 
         @Override
-        public void onLoadCanceled(EventTime eventTime, MediaSourceEventListener.LoadEventInfo loadEventInfo, MediaSourceEventListener.MediaLoadData mediaLoadData) {
+        public void onDrmSessionManagerError(@NonNull EventTime eventTime, @NonNull Exception error) {
 
         }
 
         @Override
-        public void onLoadError(EventTime eventTime, MediaSourceEventListener.LoadEventInfo loadEventInfo, MediaSourceEventListener.MediaLoadData mediaLoadData, IOException error, boolean wasCanceled) {
+        public void onDrmKeysRestored(@NonNull EventTime eventTime) {
 
         }
 
         @Override
-        public void onDownstreamFormatChanged(EventTime eventTime, MediaSourceEventListener.MediaLoadData mediaLoadData) {
+        public void onDrmKeysRemoved(@NonNull EventTime eventTime) {
 
         }
 
         @Override
-        public void onUpstreamDiscarded(EventTime eventTime, MediaSourceEventListener.MediaLoadData mediaLoadData) {
-
-        }
-
-        @Override
-        public void onMediaPeriodCreated(EventTime eventTime) {
-
-        }
-
-        @Override
-        public void onMediaPeriodReleased(EventTime eventTime) {
-
-        }
-
-        @Override
-        public void onReadingStarted(EventTime eventTime) {
-
-        }
-
-        @Override
-        public void onBandwidthEstimate(EventTime eventTime, int totalLoadTimeMs, long totalBytesLoaded, long bitrateEstimate) {
-
-        }
-
-        @Override
-        public void onSurfaceSizeChanged(EventTime eventTime, int width, int height) {
-
-        }
-
-        @Override
-        public void onMetadata(EventTime eventTime, Metadata metadata) {
-
-        }
-
-        @Override
-        public void onDecoderEnabled(EventTime eventTime, int trackType, DecoderCounters decoderCounters) {
-
-        }
-
-        @Override
-        public void onDecoderInitialized(EventTime eventTime, int trackType, String decoderName, long initializationDurationMs) {
-
-        }
-
-        @Override
-        public void onDecoderInputFormatChanged(EventTime eventTime, int trackType, Format format) {
-
-        }
-
-        @Override
-        public void onDecoderDisabled(EventTime eventTime, int trackType, DecoderCounters decoderCounters) {
-
-        }
-
-        @Override
-        public void onAudioSessionId(EventTime eventTime, int audioSessionId) {
-
-        }
-
-        @Override
-        public void onAudioAttributesChanged(EventTime eventTime, AudioAttributes audioAttributes) {
-
-        }
-
-        @Override
-        public void onVolumeChanged(EventTime eventTime, float volume) {
-
-        }
-
-        @Override
-        public void onDroppedVideoFrames(EventTime eventTime, int droppedFrames, long elapsedMs) {
-
-        }
-
-        @Override
-        public void onVideoSizeChanged(EventTime eventTime, int width, int height, int unappliedRotationDegrees, float pixelWidthHeightRatio) {
-
-        }
-
-        @Override
-        public void onRenderedFirstFrame(EventTime eventTime, @Nullable Surface surface) {
-
-        }
-
-        @Override
-        public void onDrmSessionAcquired(EventTime eventTime) {
-
-        }
-
-        @Override
-        public void onDrmKeysLoaded(EventTime eventTime) {
-
-        }
-
-        @Override
-        public void onDrmSessionManagerError(EventTime eventTime, Exception error) {
-
-        }
-
-        @Override
-        public void onDrmKeysRestored(EventTime eventTime) {
-
-        }
-
-        @Override
-        public void onDrmKeysRemoved(EventTime eventTime) {
-
-        }
-
-        @Override
-        public void onDrmSessionReleased(EventTime eventTime) {
+        public void onDrmSessionReleased(@NonNull EventTime eventTime) {
 
         }
     }
