@@ -8,7 +8,6 @@ import android.app.Notification;
 import android.app.NotificationChannel;
 import android.app.NotificationManager;
 import android.app.PendingIntent;
-import android.app.Service;
 import android.bluetooth.BluetoothA2dp;
 import android.bluetooth.BluetoothHeadset;
 import android.content.Context;
@@ -155,6 +154,8 @@ public class PlayerService extends JobIntentService implements RadioPlayer.Playe
 
     private boolean notificationIsActive = false;
 
+    final int pendingIntentFlag =  Build.VERSION.SDK_INT >= Build.VERSION_CODES.M ? PendingIntent.FLAG_IMMUTABLE : 0;
+
     void sendBroadCast(String action) {
         Intent local = new Intent();
         local.setAction(action);
@@ -334,6 +335,11 @@ public class PlayerService extends JobIntentService implements RadioPlayer.Playe
         public void warnAboutMeteredConnection(PlayerType playerType) throws RemoteException {
             PlayerService.this.warnAboutMeteredConnection(playerType);
         }
+
+        @Override
+        public boolean isNotificationActive() throws RemoteException {
+            return PlayerService.this.notificationIsActive;
+        }
     };
 
     private MediaSessionCompat.Callback mediaSessionCallback = null;
@@ -457,7 +463,7 @@ public class PlayerService extends JobIntentService implements RadioPlayer.Playe
         mediaSession.setCallback(mediaSessionCallback);
 
         Intent startActivityIntent = new Intent(itsContext.getApplicationContext(), ActivityMain.class);
-        mediaSession.setSessionActivity(PendingIntent.getActivity(itsContext.getApplicationContext(), 0, startActivityIntent, PendingIntent.FLAG_UPDATE_CURRENT));
+        mediaSession.setSessionActivity(PendingIntent.getActivity(itsContext.getApplicationContext(), 0, startActivityIntent, PendingIntent.FLAG_UPDATE_CURRENT | pendingIntentFlag));
 
         mediaSession.setFlags(MediaSessionCompat.FLAG_HANDLES_MEDIA_BUTTONS | MediaSessionCompat.FLAG_HANDLES_TRANSPORT_CONTROLS);
 
@@ -500,7 +506,7 @@ public class PlayerService extends JobIntentService implements RadioPlayer.Playe
     public int onStartCommand(Intent intent, int flags, int startId) {
         // Service could be started by external forces, e.g. when we had the last media session
         // and user presses play/pause media button.
-        PlayerServiceUtil.bind(itsContext.getApplicationContext());
+        PlayerServiceUtil.bindService(itsContext.getApplicationContext());
 
         if (currentStation == null) {
             RadioDroidApp radioDroidApp = (RadioDroidApp) getApplication();
@@ -892,15 +898,16 @@ public class PlayerService extends JobIntentService implements RadioPlayer.Playe
 
         Intent stopIntent = new Intent(itsContext, PlayerService.class);
         stopIntent.setAction(ACTION_STOP);
-        PendingIntent pendingIntentStop = PendingIntent.getService(itsContext, 0, stopIntent, 0);
+        PendingIntent pendingIntentStop = PendingIntent.getService(itsContext, 0, stopIntent, pendingIntentFlag);
+
 
         Intent nextIntent = new Intent(itsContext, PlayerService.class);
         nextIntent.setAction(ACTION_SKIP_TO_NEXT);
-        PendingIntent pendingIntentNext = PendingIntent.getService(itsContext, 0, nextIntent, 0);
+        PendingIntent pendingIntentNext = PendingIntent.getService(itsContext, 0, nextIntent, pendingIntentFlag);
 
         Intent previousIntent = new Intent(itsContext, PlayerService.class);
         previousIntent.setAction(ACTION_SKIP_TO_PREVIOUS);
-        PendingIntent pendingIntentPrevious = PendingIntent.getService(itsContext, 0, previousIntent, 0);
+        PendingIntent pendingIntentPrevious = PendingIntent.getService(itsContext, 0, previousIntent, pendingIntentFlag);
 
         PlayState currentPlayerState = radioPlayer.getPlayState();
 
@@ -915,7 +922,7 @@ public class PlayerService extends JobIntentService implements RadioPlayer.Playe
             }
         }
 
-        PendingIntent contentIntent = PendingIntent.getActivity(itsContext, 0, notificationIntent, PendingIntent.FLAG_UPDATE_CURRENT);
+        PendingIntent contentIntent = PendingIntent.getActivity(itsContext, 0, notificationIntent, PendingIntent.FLAG_UPDATE_CURRENT | pendingIntentFlag);
         NotificationCompat.Builder notificationBuilder = new NotificationCompat.Builder(itsContext, NOTIFICATION_CHANNEL_ID)
                 .setContentIntent(contentIntent)
                 .setContentTitle(theTitle)
@@ -931,7 +938,7 @@ public class PlayerService extends JobIntentService implements RadioPlayer.Playe
         if (currentPlayerState == PlayState.Playing || currentPlayerState == PlayState.PrePlaying) {
             Intent pauseIntent = new Intent(itsContext, PlayerService.class);
             pauseIntent.setAction(ACTION_PAUSE);
-            PendingIntent pendingIntentPause = PendingIntent.getService(itsContext, 0, pauseIntent, 0);
+            PendingIntent pendingIntentPause = PendingIntent.getService(itsContext, 0, pauseIntent, pendingIntentFlag);
 
             notificationBuilder.addAction(R.drawable.ic_pause_white_24dp, getString(R.string.action_pause), pendingIntentPause);
             notificationBuilder.setUsesChronometer(true)
@@ -939,7 +946,7 @@ public class PlayerService extends JobIntentService implements RadioPlayer.Playe
         } else if (currentPlayerState == PlayState.Paused || currentPlayerState == PlayState.Idle) {
             Intent resumeIntent = new Intent(itsContext, PlayerService.class);
             resumeIntent.setAction(ACTION_RESUME);
-            PendingIntent pendingIntentResume = PendingIntent.getService(itsContext, 0, resumeIntent, 0);
+            PendingIntent pendingIntentResume = PendingIntent.getService(itsContext, 0, resumeIntent, pendingIntentFlag);
 
             notificationBuilder.addAction(R.drawable.ic_play_arrow_white_24dp, getString(R.string.action_resume), pendingIntentResume);
             notificationBuilder.setUsesChronometer(false)
